# Copyright (c) 2025-2025 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

ENV_SUGGESTIONS = [
  {
    "ENV": "CPU_AFFINITY_CONF",
    "SUGGESTION_VALUE": 2,
    "REASON": "CPU 细粒度绑核",
  },
  {
    "ENV": "NPU_MEMORY_FRACTION",
    "SUGGESTION_VALUE": 0.97,
    "REASON": "NPU内存占用比例，建议逐渐调高，但是太高会引起OOM",
  },
  {
    "ENV": "TASK_QUEUE_ENABLE",
    "SUGGESTION_VALUE": 2,
    "REASON": "配置task_queue 算子下发队列优化登记，可能导致运行中NPU内存峰值上升",
  },
  {
    "ENV": "HCCL_OP_EXPANSION_MODE",
    "SUGGESTION_VALUE": "AIV",
    "REASON": "配置通信算法的编排展开位置，代表通信算法的编排展开位置在 Device侧的 AI Vector Core 计算单元"\
    "（MindIE 2.0.T3 和 MindIE 2.0.T3.1 使能 AIV 会有崩溃风险，请不要设置它）",
  },
  {
    "ENV": "HCCL_DETERMINISTIC",
    "SUGGESTION_VALUE": "false",
    "REASON": "关闭确定性计算，一般情况下无需开启确定性计算，当模型多次执行结果不同或者精度调优时，可通过此环境变量开启确定性计算进行辅助调试调优，但开启确定性计算后，算子执行时间会变慢，导致性能下降。",
    "ALLOW_UNDEFINED": True,
  },
  {
    "ENV": "HCCL_RDMA_PCIE_DIRECT_POST_NOSTRICT",
    "SUGGESTION_VALUE": "TRUE",
    "REASON": "host bound 时性能有提升 (某些局点该环境变量和内核版本升级可以二选一)",
  },
  {
    "ENV": "MINDIE_LOG_LEVEL",
    "SUGGESTION_VALUE": "ERROR",
    "REASON": "大量的日志打印是十分耗时的行为，且在正常的服务过程中，不需要这些日志",
    "ALLOW_UNDEFINED": True,
  },
  {
    "ENV": "ASCEND_GLOBAL_LOG_LEVEL",
    "SUGGESTION": 3,
    "REASON": "大量的日志打印是十分耗时的行为，且在正常的服务过程中，不需要这些日志",
    "ALLOW_UNDEFINED": True,
  },
  {
    "ENV": "ASCEND_LAUNCH_BLOCKING",
    "SUGGESTION_VALUE": "",
    "REASON": "关闭算子执行时启动同步模式（异步更快）",
  },
  {
    "ENV": "ATB_WORKSPACE_MEM_ALLOC_ALG_TYPE",
    "SUGGESTION_VALUE": 2,
    "REASON": "workspace 内存分配算法选择，可通过选择不同的算法测试workspace分配情况",
  },
  {
    "ENV": "ATB_WORKSPACE_MEM_ALLOC_GLOBAL",
    "SUGGESTION_VALUE": 1,
    "REASON": "使用全局中间tensor 内存分配算法，会对中间tensor内存进行大小计算与分配",
  },
  {
    "ENV": "PYTORCH_NPU_ALLOC_CONF",
    "SUGGESTION_VALUE": "expandable_segments:True",
    "REASON": "使能内存池扩展段功能，既虚拟内存特性；设置为True,可以优化内存碎片对内存的占用",
  },
]



# Copyright (c) 2025-2025 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from ms_performance_prechecker.prechecker.register import register_checker, cached
from ms_performance_prechecker.prechecker.register import check_result, record, CONTENT_PARTS, CheckResult
from ms_performance_prechecker.prechecker.utils import CHECK_TYPES, logger, SUGGESTION_TYPES
from ms_performance_prechecker.prechecker.env_suggestion import ENV_SUGGESTIONS


def save_env_contents(fix_pair, save_path):
    save_path = os.path.realpath(save_path)

    with open(save_path, "w") as ff:
        ff.write("ENABLE=${1-1}\n")
        ff.write('echo "ENABLE=$ENABLE"\n\n')
        ff.write('if [ "$ENABLE" = "1" ]; then\n    ')
        ff.write("\n    ".join((x[0] for x in fix_pair)) + "\n")
        ff.write('else\n    ')
        ff.write("\n    ".join((x[1] for x in fix_pair)) + "\n")
        ff.write('fi\n')
    return save_path


def version_in_black_list(version_info, black_list):
    for black_version in black_list:
        if version_info.startswith(black_version):
            return True
    return False


@register_checker()
def simple_env_checker(env_save_path, **kwargs):
    env = kwargs.get("env", {})
    fix_pair = []
    for item in ENV_SUGGESTIONS:
        env_item = item.get("ENV")
        env_value = os.getenv(env_item, "")
        env_suggest_value = item.get("SUGGESTION_VALUE", "")
        suggest_reason = item.get("REASON", "")
        version_black_list = item.get("VERSION_BLACK_LIST", [])
        allow_undefined = item.get("ALLOW_UNDEFINED", False)
        if allow_undefined and not env_value:
            check_result("env", env_item, CheckResult.OK)
            continue
        if str(env_value).lower() == str(env_suggest_value).lower():
            check_result("env", env_item, CheckResult.OK)
            continue

        logger.debug(f"{env_item}: {env_value} -> {env_suggest_value}")
        env_cmd = f"export {env_item}={env_suggest_value}" if env_suggest_value else f"unset {env_item}"

        
        check_result("env", env_item, CheckResult.ERROR,
            action=env_cmd,
            reason=suggest_reason,
        )

        pre_env = f"export {env_item}={env_value}" if env_value else f"unset {env_item}"

   
        fix_pair.append((env_cmd, pre_env))

    if not env_save_path:
        check_result("env", "SAVE ENV FILE", CheckResult.UNFINISH,
            reason="save_env setting to None/Empty",
        )
        return
    
    if len(fix_pair) == 0:
        check_result("env", "SAVE ENV FILE", CheckResult.VIP,
            action=f"None env related needs to save",
        )
        return 

    save_path = save_env_contents(fix_pair, env_save_path)
    
    check_result("env", "", CheckResult.VIP,
        action=f"环境相关改动使能：source {save_path};",
    )
    check_result("env", "", CheckResult.VIP,
        action=f"使能后恢复：source {save_path} 0",
    )





# Copyright (c) 2025-2025 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import platform
from ms_performance_prechecker.prechecker.register import register_checker, cached
from ms_performance_prechecker.prechecker.register import check_result, record, CONTENT_PARTS, CheckResult
from ms_performance_prechecker.prechecker.utils import CHECK_TYPES, SUGGESTION_TYPES
from ms_performance_prechecker.prechecker.utils import get_dict_value_by_pos, str_to_digit, logger

try:
    import acl
except ModuleNotFoundError:
    logger.warning("Import acl error, will skip getting NPU info. Install and source cann toolkit if needed")
    acl = None


DRIVER_VERSION_PATH = "/usr/local/Ascend/driver/version.info"
CPUINFO_PATH = "/proc/cpuinfo"
TRANSPARENT_HUGEPAGE_PATH = "/sys/kernel/mm/transparent_hugepage/enabled"
GOVERNOR_PATH_FORMATTER = "/sys/devices/system/cpu/cpu{core}/cpufreq/scaling_governor"


def get_cpu_info():
    import subprocess
    from shutil import which

    lscpu_path = which("lscpu")
    if not lscpu_path:
        logger.error("lscpu command not exists, will skip getting cpu info.")
        return {}

    try:
        result = subprocess.run([lscpu_path], capture_output=True, text=True, check=True, shell=False)
    except Exception as err:
        logger.error("Failed calling lscpu, will skip getting cpu info.")

    cpu_info = {}
    for line in result.stdout.splitlines():
        if ":" in line:
            key, value = line.split(":", 1)
            cpu_info[key.strip()] = value.strip()
    return cpu_info


@register_checker()
def system_info_checker(mindie_service_config, check_type, **kwargs):
    cpu_info = get_cpu_info()
    cpu_num = cpu_info.get("CPU(s)", None)
    cpu_model_name = cpu_info.get("Model name", None)
    record(f"0500 CPU 型号：{cpu_model_name}", part=CONTENT_PARTS.sys)
    record(f"0600 CPU 核心数：{cpu_num}", part=CONTENT_PARTS.sys)

    page_size = os.sysconf("SC_PAGESIZE")
    record(f"0800 页表大小：{page_size}", part=CONTENT_PARTS.sys)

    ascend_toolkit_home = os.getenv("ASCEND_TOOLKIT_HOME")
    ascend_toolkit_version_file = os.path.join(ascend_toolkit_home, "version.cfg") if ascend_toolkit_home else None
    if ascend_toolkit_version_file and os.path.exists(ascend_toolkit_version_file):
        ascend_toolkit_version = ""
        with open(ascend_toolkit_version_file) as ff:
            for line in ff.readlines():
                if "=" in line:
                    ascend_toolkit_version = line.split("=")[-1].strip()
                    break
        record(f"0100 CANN 版本：{ascend_toolkit_version[1:-1]}", part=CONTENT_PARTS.sys)

    mies_install_path = os.getenv("MIES_INSTALL_PATH")
    mindie_version_file = os.path.join(mies_install_path, "version.info") if mies_install_path else None
    if mindie_version_file and os.path.exists(mindie_version_file):
        mindie_version = ""
        with open(mindie_version_file) as ff:
            for line in ff.readlines():
                if "Ascend-mindie-service" in line and ":" in line:
                    mindie_version = line.split(":")[-1].strip()
                    break
        record(f"0200 MINDIE 版本：{mindie_version}", part=CONTENT_PARTS.sys)


@register_checker()
def linux_kernel_release_checker(mindie_service_config, check_type, **kwargs):
    target_major_version, target_minor_version = 5, 10
    target_version = ".".join([str(ii) for ii in [target_major_version, target_minor_version]])

    kernel_release = platform.release()
    logger.debug(f"Got kernel_release: {kernel_release}, suggested is {target_version}")
    record(f"0400 Linux 内核版本：{kernel_release}", part=CONTENT_PARTS.sys)

    kernel_release_split = kernel_release.split(".")
    if len(kernel_release_split) < 2:
        logger.warning(f"failed parsing kernel release version: {kernel_release}")
        return

    major_version, minor_version = str_to_digit(kernel_release_split[0]), str_to_digit(kernel_release_split[1])
    if major_version is None or minor_version is None:
        logger.warning(f"failed parsing kernel release version: {kernel_release}")
        return

    answer_kwargs = dict(
        domain="system",
        checker="内核版本", 
        result=CheckResult.ERROR,
        action=f"升级到 {target_version} 以上",
        reason="内核版本升级后以上 host bound 时性能有提升",
    )
    if major_version < target_major_version:
        check_result(**answer_kwargs)
    elif major_version == target_major_version and minor_version < target_minor_version:
        check_result(**answer_kwargs)
    else:
        check_result("system", "内核版本", CheckResult.OK)


@register_checker()
def driver_version_checker(mindie_service_config, check_type, **kwargs):
    target_major_version, target_minor_version, target_mini_version = 24, 1, 0
    target_version = ".".join([str(ii) for ii in [target_major_version, target_minor_version, target_mini_version]])

    if not os.path.exists(DRIVER_VERSION_PATH) or not os.access(DRIVER_VERSION_PATH, os.R_OK):
        logger.warning(f"{DRIVER_VERSION_PATH} not accessible")
        return

    version = ""
    with open(DRIVER_VERSION_PATH) as ff:
        for line in ff.readlines():
            if "Version=" in line:
                version = line.strip().split("=")[-1]
                break
    logger.debug(f"Got driver version: {version}, suggested is {target_version}")
    record(f"0300 驱动版本：{version}", part=CONTENT_PARTS.sys)

    version_split = version.split(".")
    if len(version_split) < 3:
        logger.warning(f"failed parsing Ascend driver version: {version}")
        return
    major_version, minor_version = str_to_digit(version_split[0]), str_to_digit(version_split[1])
    mini_version = str_to_digit(version_split[2], default_value=-1)  # value like "rc1" convert to -1
    if major_version is None or minor_version is None:
        logger.warning(f"failed parsing Ascend driver version: {version}")
        return

    answer_kwargs = dict(
        domain="system",
        checker="驱动版本", 
        result=CheckResult.ERROR,
        action=f"升级到 {target_version} 以上",
        reason="驱动版本升级后性能有提升",
    )
    if major_version < target_major_version:
        check_result(**answer_kwargs)
    elif major_version == target_major_version and minor_version < target_minor_version:
        check_result(**answer_kwargs)
    elif (
        major_version == target_major_version
        and minor_version == target_minor_version
        and mini_version < target_mini_version
    ):
        check_result(**answer_kwargs)
    else:
        check_result("system", "驱动版本", CheckResult.OK)


@register_checker()
def virtual_machine_checker(mindie_service_config, check_type, **kwargs):
    if not os.path.exists(CPUINFO_PATH) or not os.access(CPUINFO_PATH, os.R_OK):
        logger.warning(f"{CPUINFO_PATH} not accessible")
        return

    is_virtual_machine = False
    with open(CPUINFO_PATH) as ff:
        for line in ff.readlines():
            if "hypervisor" in line:
                is_virtual_machine = True
                logger.info(f"Got hypervisor info from: {CPUINFO_PATH}")
                break
    if is_virtual_machine:
        vmware_action = "启用 CPU/MMU Virtualization（ESXi 高级设置）、禁用 CPU 限制（cpuid.coresPerSocket 配置为物理核心数）"
        kvm_action = "配置 host-passthrough 模式（暴露完整 CPU 指令集）、启用多队列 virtio-net（减少网络延迟）"
        check_result("system", "可能是虚拟机", CheckResult.ERROR, 
            action=f"确定分配的 cpu 是完全体，如 VMware 中 {vmware_action}；KVM 中 {kvm_action}",
            reason="虚拟机和物理机的 cpu 核数、频率有差异会导致性能下降",
        )
    record(f"1000 是否虚拟机：{'是' if is_virtual_machine else '否'}", part=CONTENT_PARTS.sys)


@register_checker()
def transparent_hugepage_checker(mindie_service_config, check_type, **kwargs):
    if not os.path.exists(TRANSPARENT_HUGEPAGE_PATH) or not os.access(TRANSPARENT_HUGEPAGE_PATH, os.R_OK):
        logger.warning(f"{TRANSPARENT_HUGEPAGE_PATH} not accessible")
        return

    is_transparent_hugepage_enable = False
    with open(TRANSPARENT_HUGEPAGE_PATH) as ff:
        for line in ff.readlines():
            if "always" in line:
                is_transparent_hugepage_enable = True
                logger.debug(f"Got 'always' from: {TRANSPARENT_HUGEPAGE_PATH}")
                break
    if not is_transparent_hugepage_enable:
        check_result("system", "透明大页", CheckResult.ERROR,
            action=f"设置为 always：echo always > {TRANSPARENT_HUGEPAGE_PATH}",
            reason="开启透明大页，多次实验的吞吐率结果会更稳定",
        )
    record(f"0900 是否开启透明大页：{'是' if is_transparent_hugepage_enable else '否'}", part=CONTENT_PARTS.sys)


@register_checker()
def cpu_high_performance_checker(mindie_service_config, check_type, **kwargs):
    cpu_count = os.cpu_count()
    is_performances = []
    for core in range(cpu_count):
        cur_governor_path = GOVERNOR_PATH_FORMATTER.format(core=core)
        if not os.path.exists(cur_governor_path) or not os.access(cur_governor_path, os.R_OK):
            continue

        with open(cur_governor_path, "r") as ff:
            for line in ff.readlines():
                if line.strip() == "performance":
                    is_performances.append(True)
                    break
    is_cpu_all_performance_mode = len(is_performances) == cpu_count
    if not is_cpu_all_performance_mode:
        yum_cmd = "EulerOS/CentOS: yum install kernel-tools"
        apt_cmd = "Ubuntu：apt install cpufrequtils"
        run_cmd = "cpupower -c all frequency-set -g performance"
        fail_info = "如果失败可能需要在 BIOS 中开启"
        check_result("system", "CPU高性能模式", CheckResult.ERROR,
            action=f"开启 CPU 高性能模式：{run_cmd}；如果没有 cpupower 命令可以通过 {yum_cmd} 或 {apt_cmd} 安装；{fail_info}",
            reason="在相同时延约束下，TPS会有~3%的提升",
        )
    record(f"0700 CPU 是否高性能模式：{'是' if is_cpu_all_performance_mode else '否'}", part=CONTENT_PARTS.sys)



# Copyright (c) 2025-2025 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import json
import csv
from collections import namedtuple
from glob import glob

from ms_performance_prechecker.prechecker.utils import CHECK_TYPES, LOG_LEVELS, SUGGESTION_TYPES
from ms_performance_prechecker.prechecker.utils import str_ignore_case, logger, set_log_level

MIES_INSTALL_PATH = "MIES_INSTALL_PATH"
MINDIE_SERVICE_DEFAULT_PATH = "/usr/local/Ascend/mindie/latest/mindie-service"

LOG_LEVELS_LOWER = [ii.lower() for ii in LOG_LEVELS.keys()]


def read_csv(file_path):
    result = {}
    with open(file_path, mode="r", newline="", encoding="utf-8") as ff:
        for row in csv.DictReader(ff):
            for kk, vv in row.items():
                result.setdefault(kk, []).append(vv)
    return result


def read_json(file_path):
    with open(file_path) as ff:
        result = json.load(ff)
    return result


def read_csv_or_json(file_path):
    logger.debug(f"{file_path = }")
    if not file_path or not os.path.exists(file_path):
        return None
    if file_path.endswith(".json"):
        return read_json(file_path)
    if file_path.endswith(".csv"):
        return read_csv(file_path)
    return None


def get_next_dict_item(dict_value):
    return dict([next(iter(dict_value.items()))])


""" parse_mindie_server_config """


def parse_mindie_server_config(mindie_service_path=None):
    logger.debug("mindie_service_config:")
    if mindie_service_path is None:
        mindie_service_path = os.getenv(MIES_INSTALL_PATH, MINDIE_SERVICE_DEFAULT_PATH)
    if not os.path.exists(mindie_service_path):
        logger.warning(f"mindie config.json: {mindie_service_path} not exists, will skip related checkers")
        return None

    mindie_service_config = read_csv_or_json(os.path.join(mindie_service_path, "conf", "config.json"))
    logger.debug(
        f"mindie_service_config: {get_next_dict_item(mindie_service_config) if mindie_service_config else None}"
    )
    return mindie_service_config


""" prechecker """


def run_precheck(mindie_service_path=None, check_type=CHECK_TYPES.deepseek,
    env_save_path="ms_performance_prechecker_env.sh", **kwargs):
    import ms_performance_prechecker.prechecker
    from ms_performance_prechecker.prechecker.register import REGISTRY, ANSWERS, CONTENTS, CONTENT_PARTS

    mindie_service_config = parse_mindie_server_config(mindie_service_path)
    logger.debug("")
    logger.debug("<think>")
    for name, checker in REGISTRY.items():
        logger.debug(name)
        checker(mindie_service_config=mindie_service_config, check_type=check_type,
            env_save_path=env_save_path, **kwargs)
    logger.debug("</think>")

    if CONTENTS.get(CONTENT_PARTS.sys, None):
        sorted_contents = [ii.split(" ", 1)[-1] for ii in sorted(CONTENTS[CONTENT_PARTS.sys])]
        sys_info = "系统信息：\n\n    " + "\n    ".join(sorted_contents) + "\n"
        logger.info(sys_info)

    logger.info("本工具提供的为经验建议，实际效果与具体的环境/场景有关，建议以实测为准")


def arg_parse(argv):
    import argparse

    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        "-t",
        "--check_type",
        type=str_ignore_case,
        default=CHECK_TYPES.deepseek,
        choices=CHECK_TYPES,
        help="check type",
    )
    parser.add_argument(
        "-s",
        "--save_env",
        default="ms_performance_prechecker_env.sh",
        help="Save env changes as a file which could be applied directly.",
    )
    parser.add_argument("-l", "--log_level", default="info", choices=LOG_LEVELS_LOWER, help="specify log level.")
    return parser.parse_known_args(argv)[0]


def main():
    import sys

    args = arg_parse(sys.argv)
    set_log_level(args.log_level)
    run_precheck(None, args.check_type, args.save_env, args=args)


if __name__ == "__main__":
    main()



# Copyright (c) 2025-2025 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from collections import namedtuple

CHECK_TYPES = namedtuple("CHECK_TYPES", ["basic", "deepseek"])("basic", "deepseek")
_SUGGESTION_TYPES = ["env", "system", "config"]
SUGGESTION_TYPES = namedtuple("SUGGESTION_TYPES", _SUGGESTION_TYPES)(*_SUGGESTION_TYPES)

LOG_LEVELS = {
    "debug": logging.DEBUG,
    "info": logging.INFO,
    "warning": logging.WARNING,
    "error": logging.ERROR,
    "fatal": logging.FATAL,
    "critical": logging.CRITICAL,
}


def str_ignore_case(value):
    return value.lower().replace("_", "").replace("-", "")


def str_to_digit(input_str, default_value=None):
    if not input_str.replace(".", "", 1).isdigit():
        return default_value
    return float(input_str) if "." in input_str else int(input_str)


def walk_dict(data, parent_key=""):
    if isinstance(data, dict):
        for key, value in data.items():
            if not isinstance(value, (dict, tuple, list)):
                yield key, value, parent_key
            else:
                new_key = f"{parent_key}.{key}" if parent_key else key
                yield from walk_dict(value, new_key)
    elif isinstance(data, (tuple, list)):
        for index, item in enumerate(data):
            if not isinstance(item, (dict, tuple, list)):
                yield key, item, parent_key
            else:
                new_key = f"{parent_key}.{index}" if parent_key else index
                yield from walk_dict(item, new_key)


def get_dict_value_by_pos(dict_value, target_pos):
    cur = dict_value
    for kk in target_pos.split(":"):
        if not cur:
            cur = None
            break
        if isinstance(cur, list) and str.isdigit(kk):
            cur = cur[int(kk)]
        elif kk in cur:
            cur = cur[kk]
        else:
            cur = None
            break
    return cur


def set_log_level(level="info"):
    if level.lower() in LOG_LEVELS:
        logger.setLevel(LOG_LEVELS.get(level.lower()))
    else:
        logger.warning("Set %s log level failed.", level)


def set_logger(msit_logger):
    msit_logger.propagate = False
    msit_logger.setLevel(logging.INFO)
    if not msit_logger.handlers:
        stream_handler = logging.StreamHandler()
        formatter = logging.Formatter("%(name)s - %(levelname)s - %(message)s")
        stream_handler.setFormatter(formatter)
        msit_logger.addHandler(stream_handler)


def get_version_info(mindie_service_path):
    if mindie_service_path is None:
        mindie_service_path = os.getenv(MIES_INSTALL_PATH, MINDIE_SERVICE_DEFAULT_PATH)

    version_path = os.path.join(mindie_service_path, "version.info")

    if not os.path.exists(version_path): 
        return {}
    
    version_info = {}
    with open(version_path) as f:
        for line in f:
            line_split = line.split(":")
            key, value = line_split[0], line_split[-1]
            version_info[key.strip()] = value.strip()

    return version_info


logger = logging.getLogger("ms_performance_prechecker_logger")
set_logger(logger)



# Copyright (c) 2025-2025 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
from collections import namedtuple
from ms_performance_prechecker.prechecker.utils import CHECK_TYPES, SUGGESTION_TYPES, logger

# 创建一个全局的注册表，注册为分析函数
REGISTRY = {}

ANSWERS = {ii: {} for ii in SUGGESTION_TYPES}
CONTENT_PARTS = namedtuple("CONTENT_PARTS", ["before", "after", "sys"])("before", "after", "sys")
CONTENTS = {}  # Will save to file in the end


def register_checker(analyze_name=None):
    def decorator(func):
        name = analyze_name if analyze_name is not None else func.__name__
        REGISTRY[name] = func
        return func

    return decorator


def cached():
    # 缓存函数结果，反正所有输入都是一样的
    cache = {}

    def decorator(func):
        name = func.__name__

        def wrapper(*args, **kwargs):
            if name in cache:
                return cache[name]
            result = func(*args, **kwargs)
            cache[name] = result
            return result

        return wrapper

    return decorator


def answer(suggesion_type=None, suggesion_item=None, action=None, reason=""):
    ANSWERS[suggesion_type].setdefault(suggesion_item, []).append((action, reason))


def record(content, part=CONTENT_PARTS.after):
    CONTENTS.setdefault(part, []).append(content)


CheckResult = Enum('CheckResult', ['OK', 'UNFINISH', 'WARN', 'ERROR', "VIP"])


def check_result(domain, checker, result=None, action=None, reason=None):
    color_and_text = {CheckResult.OK: ('\033[92m', "OK"),
                       CheckResult.UNFINISH: ('\033[93m', "UNFINISH"),
                       CheckResult.WARN: ('\033[93m', "WARN"),
                       CheckResult.ERROR: ('\033[91m', "NOK"),
                       CheckResult.VIP: ('\033[94m', action)}

    if result is None:
        color, text = '\033[97m', ""
    else:
        color, text = color_and_text.get(result, ('\033[97m', ""))
    logger.info(f"{domain} {checker} ... {color} {text} \033[0m")
    if action is not None and result != CheckResult.VIP:
        logger.info(f"    * {action}")
    if reason is not None:
        logger.info(f"    * {reason}")
